﻿using Jiguang.JPush;
using Jiguang.JPush.Model;
using Newtonsoft.Json.Linq;
using PushPlatform.Common.Model;
using PushPlatform.Common.Util;
using System;
using System.Collections.Generic;
using System.Text;

namespace PushInfo.Core.BLL
{
    public class JiguangPushMessage
    {
        private readonly JPushClient _client;

        public JiguangPushMessage(string appkey, string masterSecret)
        {
            _client = new JPushClient(appkey, masterSecret);
        }

        /// <summary>
        /// 发送推送通知
        /// 参考文档：https://docs.jiguang.cn/jpush/server/push/rest_api_v3_push
        /// </summary>
        /// <param name="audience">推送目标，支持别名、标签、注册 ID、分群</param>
        /// <param name="title">通知标题</param>
        /// <param name="alert">通知内容</param>
        /// <param name="intent">Android指定跳转页面，如果extras参数不为空，扩展字段将拼接到intent中</param>
        /// <param name="extras">扩展字段</param>
        /// <param name="isProd">iOS App是否上架，上架之前使用开发证书，上架之后使用生产证书来推送，并且可以将生产证书用于开发环境</param>
        /// <returns></returns>
        public APIResult<string> SendPush(Audience audience, string title, string alert, string intent, Dictionary<string, object> extras = null, bool isProd = false)
        {
            APIResult<string> result = new APIResult<string>();
            result.Result = 1;
            try
            {
                if (!string.IsNullOrWhiteSpace(intent))
                    intent = ProcessAndroidIntent(intent, extras);

                var pushPayload = new PushPayload
                {
                    Platform = "all", // 所有平台
                    Audience = audience
                };
                var notification = new Notification()
                {
                    Android = new Android
                    {
                        Alert = alert,
                        Title = title,
                        Extras = extras,
                        Indent = new Dictionary<string, object> { { "url", intent } }
                    },
                    IOS = new IOS
                    {
                        Alert = new JObject
                        {
                            { "title", title },
                            { "body", alert }
                        },
                        ContentAvailable = false,
                        Extras = extras
                    }
                };
                pushPayload.Notification = notification;
                pushPayload.Options = new Options
                {
                    IsApnsProduction = isProd
                };

                var response = _client.SendPush(pushPayload);
                result.Msg = response.Content;
                if (response.StatusCode != System.Net.HttpStatusCode.OK)
                    result.Result = 0;
            }
            catch (Exception ex)
            {
                LogHelper.Error(ex);
                result.Result = 0;
                result.code = 1;
            }
            return result;
        }

        private string ProcessAndroidIntent(string intent, Dictionary<string, object> extras)
        {
            if (string.IsNullOrWhiteSpace(intent) || extras == null || extras.Count == 0)
                return intent;
            var param = new StringBuilder();
            foreach (var item in extras)
            {
                var valueType = item.Value.GetType();
                if (valueType == typeof(string))
                    param.AppendFormat("S.{0}={1};", item.Key, item.Value);
                else if (IsNumericType(item.Value))
                    param.AppendFormat("i.{0}={1};", item.Key, item.Value);
            }
            if (param.Length > 0)
                return $"{intent.Substring(0, intent.LastIndexOf(";"))};{param}end";
            return intent;
        }

        private bool IsNumericType(object o)
        {
            switch (Type.GetTypeCode(o.GetType()))
            {
                case TypeCode.Byte:
                case TypeCode.SByte:
                case TypeCode.UInt16:
                case TypeCode.UInt32:
                case TypeCode.UInt64:
                case TypeCode.Int16:
                case TypeCode.Int32:
                case TypeCode.Int64:
                    return true;
                default:
                    return false;
            }
        }
    }
}
